@@ -3,8 +3,6 @@ var annotated_dup =
     [ "tvg", null, [
       [ "Point", "structtvg_1_1Point.html", null ],
       [ "Matrix", "structtvg_1_1Matrix.html", null ],
-      [ "Vertex", "structtvg_1_1Vertex.html", null ],
-      [ "Polygon", "structtvg_1_1Polygon.html", null ],
       [ "Paint", "classtvg_1_1Paint.html", "classtvg_1_1Paint" ],
       [ "Fill", "classtvg_1_1Fill.html", "classtvg_1_1Fill" ],
       [ "Canvas", "classtvg_1_1Canvas.html", "classtvg_1_1Canvas" ],
@@ -20,7 +18,6 @@ var annotated_dup =
       [ "Initializer", "classtvg_1_1Initializer.html", null ],
       [ "Animation", "classtvg_1_1Animation.html", "classtvg_1_1Animation" ],
       [ "Saver", "classtvg_1_1Saver.html", "classtvg_1_1Saver" ],
-      [ "Accessor", "classtvg_1_1Accessor.html", "classtvg_1_1Accessor" ],
-      [ "LottieAnimation", "classtvg_1_1LottieAnimation.html", "classtvg_1_1LottieAnimation" ]
+      [ "Accessor", "classtvg_1_1Accessor.html", "classtvg_1_1Accessor" ]
     ] ]
 ];
\ No newline at end of file
