@@ -28,6 +28,7 @@
 #include "tvgCommon.h"
 #include "tvgRender.h"
 #include "tvgLottieProperty.h"
+#include "tvgLottieRenderPooler.h"
 
 
 struct LottieComposition;
@@ -75,10 +76,106 @@ struct LottieStroke
     StrokeJoin join = StrokeJoin::Round;
 };
 
+struct LottieEffect
+{
+    enum Type : uint8_t {Tint = 20, Fill, Stroke, Tritone, DropShadow = 25, GaussianBlur = 29};
+
+    virtual ~LottieEffect() {}
+    Type type;
+    bool enable = false;
+};
+
+struct LottieFxFill : LottieEffect
+{
+    //LottieInteger mask;
+    //LottieInteger allMask;
+    LottieColor color;
+    //LottieInteger invert;
+    //LottieSlider hFeather;
+    //LottieSlider vFeather;
+    LottieFloat opacity;
+
+    LottieFxFill()
+    {
+        type = LottieEffect::Fill;
+    }
+};
+
+struct LottieFxStroke : LottieEffect
+{
+    LottieInteger mask;
+    LottieInteger allMask;
+    //LottieInteger sequential;
+    LottieColor color;
+    LottieFloat size;
+    //LottieFloat hardness;    //should support with the blurness?
+    LottieFloat opacity;
+    LottieFloat begin;
+    LottieFloat end;
+    //LottieFloat space;
+    //LottieInteger style;
+
+    LottieFxStroke()
+    {
+        type = LottieEffect::Stroke;
+    }
+};
+
+struct LottieFxTint : LottieEffect
+{
+    LottieColor black;
+    LottieColor white;
+    LottieFloat intensity;
+
+    LottieFxTint()
+    {
+        type = LottieEffect::Tint;
+    }
+};
+
+struct LottieFxTritone : LottieEffect
+{
+    LottieColor bright;
+    LottieColor midtone;
+    LottieColor dark;
+
+    LottieFxTritone()
+    {
+        type = LottieEffect::Tritone;
+    }
+};
+
+struct LottieFxDropShadow : LottieEffect
+{
+    LottieColor color;
+    LottieFloat opacity = 0;
+    LottieFloat angle = 0.0f;
+    LottieFloat distance = 0.0f;
+    LottieFloat blurness = 0.0f;
+
+    LottieFxDropShadow()
+    {
+        type = LottieEffect::DropShadow;
+    }
+};
+
+struct LottieFxGaussianBlur : LottieEffect
+{
+    LottieFloat blurness = 0.0f;
+    LottieInteger direction = 0;
+    LottieInteger wrap = 0;
+
+    LottieFxGaussianBlur()
+    {
+        type = LottieEffect::GaussianBlur;
+    }
+};
+
 
 struct LottieMask
 {
     LottiePathSet pathset;
+    LottieFloat expand = 0.0f;
     LottieOpacity opacity = 255;
     CompositeMethod method;
     bool inverse = false;
@@ -105,19 +202,21 @@ struct LottieObject
         Trimpath,
         Text,
         Repeater,
-        RoundedCorner
+        RoundedCorner,
+        OffsetPath
     };
 
     virtual ~LottieObject()
     {
     }
 
-    virtual void override(LottieProperty* prop)
+    virtual void override(LottieProperty* prop, bool shallow, bool byDefault)
     {
         TVGERR("LOTTIE", "Unsupported slot type");
     }
 
     virtual bool mergeable() { return false; }
+    virtual LottieProperty* property(uint16_t ix) { return nullptr; }
 
     unsigned long id = 0;
     Type type;
@@ -148,6 +247,49 @@ struct LottieGlyph
 };
 
 
+struct LottieTextRange
+{
+    enum Based : uint8_t { Chars = 1, CharsExcludingSpaces, Words, Lines };
+    enum Shape : uint8_t { Square = 1, RampUp, RampDown, Triangle, Round, Smooth };
+    enum Unit : uint8_t { Percent = 1, Index };
+
+    ~LottieTextRange()
+    {
+        free(interpolator);
+    }
+
+    struct {
+        LottieColor fillColor = RGB24{255, 255, 255};
+        LottieColor strokeColor = RGB24{255, 255, 255};
+        LottiePosition position = Point{0, 0};
+        LottiePoint scale = Point{100, 100};
+        LottieFloat letterSpacing = 0.0f;
+        LottieFloat lineSpacing = 0.0f;
+        LottieFloat strokeWidth = 0.0f;
+        LottieFloat rotation = 0.0f;
+        LottieOpacity fillOpacity = 255;
+        LottieOpacity strokeOpacity = 255;
+        LottieOpacity opacity = 255;
+    } style;
+
+    LottieFloat offset = 0.0f;
+    LottieFloat maxEase = 0.0f;
+    LottieFloat minEase = 0.0f;
+    LottieFloat maxAmount = 0.0f;
+    LottieFloat smoothness = 0.0f;
+    LottieFloat start = 0.0f;
+    LottieFloat end = FLT_MAX;
+    LottieInterpolator* interpolator = nullptr;
+    Based based = Chars;
+    Shape shape = Square;
+    Unit rangeUnit = Percent;
+    uint8_t random = 0;
+    bool expressible = false;
+
+    float factor(float frameNo, float totalLen, float idx);
+};
+
+
 struct LottieFont
 {
     enum Origin : uint8_t { Local = 0, CssURL, ScriptURL, FontURL, Embedded };
@@ -180,22 +322,40 @@ struct LottieMarker
     }
 };
 
-struct LottieText : LottieObject
+struct LottieText : LottieObject, LottieRenderPooler<tvg::Shape>
 {
-    void prepare()
+    struct AlignOption
+    {
+        enum Group : uint8_t { Chars = 1, Word = 2, Line = 3, All = 4 };
+        Group grouping = Chars;
+        LottiePoint anchor{};
+    } alignOption;
+
+    LottieText()
     {
         LottieObject::type = LottieObject::Text;
     }
 
-    void override(LottieProperty* prop) override
+    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
     {
-        this->doc = *static_cast<LottieTextDoc*>(prop);
-        this->prepare();
+        if (byDefault) doc.release();
+        doc.copy(*static_cast<LottieTextDoc*>(prop), shallow);
+    }
+
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (doc.ix == ix) return &doc;
+        return nullptr;
     }
 
     LottieTextDoc doc;
     LottieFont* font;
-    LottieFloat spacing = 0.0f;  //letter spacing
+    Array<LottieTextRange*> ranges;
+
+    ~LottieText()
+    {
+        for (auto r = ranges.begin(); r < ranges.end(); ++r) delete(*r);
+    }
 };
 
 
@@ -203,7 +363,7 @@ struct LottieTrimpath : LottieObject
 {
     enum Type : uint8_t { Simultaneous = 1, Individual = 2 };
 
-    void prepare()
+    LottieTrimpath()
     {
         LottieObject::type = LottieObject::Trimpath;
     }
@@ -214,6 +374,14 @@ struct LottieTrimpath : LottieObject
         return false;
     }
 
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (start.ix == ix) return &start;
+        if (end.ix == ix) return &end;
+        if (offset.ix == ix) return &offset;
+        return nullptr;
+    }
+
     void segment(float frameNo, float& start, float& end, LottieExpressions* exps);
 
     LottieFloat start = 0.0f;
@@ -223,33 +391,49 @@ struct LottieTrimpath : LottieObject
 };
 
 
-struct LottieShape : LottieObject
+struct LottieShape : LottieObject, LottieRenderPooler<tvg::Shape>
 {
+    bool clockwise = true;   //clockwise or counter-clockwise
+
     virtual ~LottieShape() {}
-    uint8_t direction = 0;   //0: clockwise, 2: counter-clockwise, 3: xor(?)
 
     bool mergeable() override
     {
         return true;
     }
+
+    LottieShape(LottieObject::Type type)
+    {
+        LottieObject::type = type;
+    }
 };
 
 
 struct LottieRoundedCorner : LottieObject
 {
-    void prepare()
+    LottieRoundedCorner()
     {
         LottieObject::type = LottieObject::RoundedCorner;
     }
+
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (radius.ix == ix) return &radius;
+        return nullptr;
+    }
+
     LottieFloat radius = 0.0f;
 };
 
 
 struct LottiePath : LottieShape
 {
-    void prepare()
+    LottiePath() : LottieShape(LottieObject::Path) {}
+
+    LottieProperty* property(uint16_t ix) override
     {
-        LottieObject::type = LottieObject::Path;
+        if (pathset.ix == ix) return &pathset;
+        return nullptr;
     }
 
     LottiePathSet pathset;
@@ -258,9 +442,14 @@ struct LottiePath : LottieShape
 
 struct LottieRect : LottieShape
 {
-    void prepare()
+    LottieRect() : LottieShape(LottieObject::Rect) {}
+
+    LottieProperty* property(uint16_t ix) override
     {
-        LottieObject::type = LottieObject::Rect;
+        if (position.ix == ix) return &position;
+        if (size.ix == ix) return &size;
+        if (radius.ix == ix) return &radius;
+        return nullptr;
     }
 
     LottiePosition position = Point{0.0f, 0.0f};
@@ -273,9 +462,18 @@ struct LottiePolyStar : LottieShape
 {
     enum Type : uint8_t {Star = 1, Polygon};
 
-    void prepare()
+    LottiePolyStar() : LottieShape(LottieObject::Polystar) {}
+
+    LottieProperty* property(uint16_t ix) override
     {
-        LottieObject::type = LottieObject::Polystar;
+        if (position.ix == ix) return &position;
+        if (innerRadius.ix == ix) return &innerRadius;
+        if (outerRadius.ix == ix) return &outerRadius;
+        if (innerRoundness.ix == ix) return &innerRoundness;
+        if (outerRoundness.ix == ix) return &outerRoundness;
+        if (rotation.ix == ix) return &rotation;
+        if (ptsCnt.ix == ix) return &ptsCnt;
+        return nullptr;
     }
 
     LottiePosition position = Point{0.0f, 0.0f};
@@ -291,9 +489,13 @@ struct LottiePolyStar : LottieShape
 
 struct LottieEllipse : LottieShape
 {
-    void prepare()
+    LottieEllipse() : LottieShape(LottieObject::Ellipse) {}
+
+    LottieProperty* property(uint16_t ix) override
     {
-        LottieObject::type = LottieObject::Ellipse;
+        if (position.ix == ix) return &position;
+        if (size.ix == ix) return &size;
+        return nullptr;
     }
 
     LottiePosition position = Point{0.0f, 0.0f};
@@ -321,7 +523,7 @@ struct LottieTransform : LottieObject
         delete(rotationEx);
     }
 
-    void prepare()
+    LottieTransform()
     {
         LottieObject::type = LottieObject::Transform;
     }
@@ -332,6 +534,49 @@ struct LottieTransform : LottieObject
         return false;
     }
 
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (position.ix == ix) return &position;
+        if (rotation.ix == ix) return &rotation;
+        if (scale.ix == ix) return &scale;
+        if (anchor.ix == ix) return &anchor;
+        if (opacity.ix == ix) return &opacity;
+        if (skewAngle.ix == ix) return &skewAngle;
+        if (skewAxis.ix == ix) return &skewAxis;
+        if (coords) {
+            if (coords->x.ix == ix) return &coords->x;
+            if (coords->y.ix == ix) return &coords->y;
+        }
+        return nullptr;
+    }
+
+    void override(LottieProperty* prop, bool shallow, bool byDefault) override
+    {
+        switch (prop->type) {
+            case LottieProperty::Type::Position: {
+                if (byDefault) position.release();
+                position.copy(*static_cast<LottiePosition*>(prop), shallow);
+                break;
+            }
+            case LottieProperty::Type::Float: {
+                if (byDefault) rotation.release();
+                rotation.copy(*static_cast<LottieFloat*>(prop), shallow);
+                break;
+            }
+            case LottieProperty::Type::Point: {
+                if (byDefault) scale.release();
+                scale.copy(*static_cast<LottiePoint*>(prop), shallow);
+                break;
+            }
+            case LottieProperty::Type::Opacity: {
+                if (byDefault) opacity.release();
+                opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
+                break;
+            }
+            default: break;
+        }
+    }
+
     LottiePosition position = Point{0.0f, 0.0f};
     LottieFloat rotation = 0.0f;           //z rotation
     LottiePoint scale = Point{100.0f, 100.0f};
@@ -349,35 +594,58 @@ struct LottieSolid : LottieObject
 {
     LottieColor color = RGB24{255, 255, 255};
     LottieOpacity opacity = 255;
+
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (color.ix == ix) return &color;
+        if (opacity.ix == ix) return &opacity;
+        return nullptr;
+    }
 };
 
 
 struct LottieSolidStroke : LottieSolid, LottieStroke
 {
-    void prepare()
+    LottieSolidStroke()
     {
         LottieObject::type = LottieObject::SolidStroke;
     }
 
-    void override(LottieProperty* prop) override
+    LottieProperty* property(uint16_t ix) override
     {
-        this->color = *static_cast<LottieColor*>(prop);
-        this->prepare();
+        if (width.ix == ix) return &width;
+        if (dashattr) {
+            if (dashattr->value[0].ix == ix) return &dashattr->value[0];
+            if (dashattr->value[1].ix == ix) return &dashattr->value[1];
+            if (dashattr->value[2].ix == ix) return &dashattr->value[2];
+        }
+        return LottieSolid::property(ix);
+    }
+
+    void override(LottieProperty* prop, bool shallow, bool byDefault) override
+    {
+        if (byDefault) color.release();
+        color.copy(*static_cast<LottieColor*>(prop), shallow);
     }
 };
 
 
 struct LottieSolidFill : LottieSolid
 {
-    void prepare()
+    LottieSolidFill()
     {
         LottieObject::type = LottieObject::SolidFill;
     }
 
-    void override(LottieProperty* prop) override
+    void override(LottieProperty* prop, bool shallow, bool byDefault) override
     {
-        this->color = *static_cast<LottieColor*>(prop);
-        this->prepare();
+        if (prop->type == LottieProperty::Type::Opacity) {
+            if (byDefault) opacity.release();
+            opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
+        } else if (prop->type == LottieProperty::Type::Color) {
+            if (byDefault) color.release();
+            color.copy(*static_cast<LottieColor*>(prop), shallow);
+        }
     }
 
     FillRule rule = FillRule::Winding;
@@ -389,19 +657,39 @@ struct LottieGradient : LottieObject
     bool prepare()
     {
         if (!colorStops.populated) {
+            auto count = colorStops.count;  //colorstop count can be modified after population
             if (colorStops.frames) {
                 for (auto v = colorStops.frames->begin(); v < colorStops.frames->end(); ++v) {
-                    colorStops.count = populate(v->value);
+                    colorStops.count = populate(v->value, count);
                 }
             } else {
-                colorStops.count = populate(colorStops.value);
+                colorStops.count = populate(colorStops.value, count);
             }
+            colorStops.populated = true;
         }
         if (start.frames || end.frames || height.frames || angle.frames || opacity.frames || colorStops.frames) return true;
         return false;
     }
 
-    uint32_t populate(ColorStop& color);
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (start.ix == ix) return &start;
+        if (end.ix == ix) return &end;
+        if (height.ix == ix) return &height;
+        if (angle.ix == ix) return &angle;
+        if (opacity.ix == ix) return &opacity;
+        if (colorStops.ix == ix) return &colorStops;
+        return nullptr;
+    }
+
+    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
+    {
+        if (byDefault) colorStops.release();
+        colorStops.copy(*static_cast<LottieColorStop*>(prop), shallow);
+        prepare();
+    }
+
+    uint32_t populate(ColorStop& color, size_t count);
     Fill* fill(float frameNo, LottieExpressions* exps);
 
     LottiePoint start = Point{0.0f, 0.0f};
@@ -416,16 +704,9 @@ struct LottieGradient : LottieObject
 
 struct LottieGradientFill : LottieGradient
 {
-    void prepare()
+    LottieGradientFill()
     {
         LottieObject::type = LottieObject::GradientFill;
-        LottieGradient::prepare();
-    }
-
-    void override(LottieProperty* prop) override
-    {
-        this->colorStops = *static_cast<LottieColorStop*>(prop);
-        this->prepare();
     }
 
     FillRule rule = FillRule::Winding;
@@ -434,47 +715,60 @@ struct LottieGradientFill : LottieGradient
 
 struct LottieGradientStroke : LottieGradient, LottieStroke
 {
-    void prepare()
+    LottieGradientStroke()
     {
         LottieObject::type = LottieObject::GradientStroke;
-        LottieGradient::prepare();
     }
 
-    void override(LottieProperty* prop) override
+    LottieProperty* property(uint16_t ix) override
     {
-        this->colorStops = *static_cast<LottieColorStop*>(prop);
-        this->prepare();
+        if (width.ix == ix) return &width;
+        if (dashattr) {
+            if (dashattr->value[0].ix == ix) return &dashattr->value[0];
+            if (dashattr->value[1].ix == ix) return &dashattr->value[1];
+            if (dashattr->value[2].ix == ix) return &dashattr->value[2];
+        }
+        return LottieGradient::property(ix);
     }
 };
 
 
-struct LottieImage : LottieObject
+struct LottieImage : LottieObject, LottieRenderPooler<tvg::Picture>
 {
-    union {
-        char* b64Data = nullptr;
-        char* path;
-    };
-    char* mimeType = nullptr;
-    uint32_t size = 0;
+    LottieBitmap data;
 
-    Picture* picture = nullptr;   //tvg render data
-
-    ~LottieImage();
-
-    void prepare()
+    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
     {
-        LottieObject::type = LottieObject::Image;
+        if (byDefault) data.release();
+        data.copy(*static_cast<LottieBitmap*>(prop), shallow);
+        update();
     }
+
+    void prepare();
+    void update();
 };
 
 
 struct LottieRepeater : LottieObject
 {
-    void prepare()
+    LottieRepeater()
     {
         LottieObject::type = LottieObject::Repeater;
     }
 
+    LottieProperty* property(uint16_t ix) override
+    {
+        if (copies.ix == ix) return &copies;
+        if (offset.ix == ix) return &offset;
+        if (position.ix == ix) return &position;
+        if (rotation.ix == ix) return &rotation;
+        if (scale.ix == ix) return &scale;
+        if (anchor.ix == ix) return &anchor;
+        if (startOpacity.ix == ix) return &startOpacity;
+        if (endOpacity.ix == ix) return &endOpacity;
+        return nullptr;
+    }
+
     LottieFloat copies = 0.0f;
     LottieFloat offset = 0.0f;
 
@@ -489,8 +783,23 @@ struct LottieRepeater : LottieObject
 };
 
 
-struct LottieGroup : LottieObject
+struct LottieOffsetPath : LottieObject
 {
+    LottieOffsetPath()
+    {
+        LottieObject::type = LottieObject::OffsetPath;
+    }
+
+    LottieFloat offset = 0.0f;
+    LottieFloat miterLimit = 4.0f;
+    StrokeJoin join = StrokeJoin::Miter;
+};
+
+
+struct LottieGroup : LottieObject, LottieRenderPooler<tvg::Shape>
+{
+    LottieGroup();
+
     virtual ~LottieGroup()
     {
         for (auto p = children.begin(); p < children.end(); ++p) delete(*p);
@@ -513,13 +822,14 @@ struct LottieGroup : LottieObject
         return nullptr;
     }
 
-    Scene* scene = nullptr;               //tvg render data
+    Scene* scene = nullptr;
     Array<LottieObject*> children;
 
-    bool reqFragment = false;   //requirment to fragment the render context
-    bool buildDone = false;     //completed in building the composition.
-    bool allowMerge = true;     //if this group is consisted of simple (transformed) shapes.
-    bool trimpath = false;      //this group has a trimpath.
+    bool reqFragment : 1;   //requirement to fragment the render context
+    bool buildDone : 1;     //completed in building the composition.
+    bool trimpath : 1;      //this group has a trimpath.
+    bool visible : 1;       //this group has visible contents.
+    bool allowMerge : 1;    //if this group is consisted of simple (transformed) shapes.
 };
 
 
@@ -538,18 +848,20 @@ struct LottieLayer : LottieGroup
 
     bool mergeable() override { return false; }
 
-    void prepare();
-    float remap(float frameNo, LottieExpressions* exp);
+    void prepare(RGB24* color = nullptr);
+    float remap(LottieComposition* comp, float frameNo, LottieExpressions* exp);
 
     char* name = nullptr;
     LottieLayer* parent = nullptr;
     LottieFloat timeRemap = 0.0f;
-    LottieComposition* comp = nullptr;
+    LottieLayer* comp = nullptr;  //Precompositor, current layer is belonges.
     LottieTransform* transform = nullptr;
     Array<LottieMask*> masks;
-    RGB24 color;  //used by Solid layer
+    Array<LottieEffect*> effects;
     LottieLayer* matteTarget = nullptr;
 
+    LottieRenderPooler<tvg::Shape> statical;  //static pooler for solid fill and clipper
+
     float timeStretch = 1.0f;
     float w = 0.0f, h = 0.0f;
     float inFrame = 0.0f;
@@ -572,9 +884,20 @@ struct LottieLayer : LottieGroup
     bool autoOrient = false;
     bool matteSrc = false;
 
+    LottieLayer* layerById(unsigned long id)
+    {
+        for (auto child = children.begin(); child < children.end(); ++child) {
+            if ((*child)->type != LottieObject::Type::Layer) continue;
+            auto layer = static_cast<LottieLayer*>(*child);
+            if (layer->id == id) return layer;
+        }
+        return nullptr;
+    }
+
     LottieLayer* layerByIdx(int16_t idx)
     {
         for (auto child = children.begin(); child < children.end(); ++child) {
+            if ((*child)->type != LottieObject::Type::Layer) continue;
             auto layer = static_cast<LottieLayer*>(*child);
             if (layer->idx == idx) return layer;
         }
@@ -590,7 +913,7 @@ struct LottieSlot
         LottieProperty* prop;
     };
 
-    void assign(LottieObject* target);
+    void assign(LottieObject* target, bool byDefault);
     void reset();
 
     LottieSlot(char* sid, LottieObject* obj, LottieProperty::Type type) : sid(sid), type(type)
@@ -601,7 +924,7 @@ struct LottieSlot
     ~LottieSlot()
     {
         free(sid);
-        if (!overriden) return;
+        if (!overridden) return;
         for (auto pair = pairs.begin(); pair < pairs.end(); ++pair) {
             delete(pair->prop);
         }
@@ -610,7 +933,7 @@ struct LottieSlot
     char* sid;
     Array<Pair> pairs;
     LottieProperty::Type type;
-    bool overriden = false;
+    bool overridden = false;
 };
 
 
@@ -632,30 +955,12 @@ struct LottieComposition
 
     float timeAtFrame(float frameNo)
     {
-        return (frameNo - startFrame) / frameRate;
+        return (frameNo - root->inFrame) / frameRate;
     }
 
     float frameCnt() const
     {
-        return endFrame - startFrame;
-    }
-
-    LottieLayer* layerById(unsigned long id)
-    {
-        for (auto child = root->children.begin(); child < root->children.end(); ++child) {
-            auto layer = static_cast<LottieLayer*>(*child);
-            if (layer->id == id) return layer;
-        }
-        return nullptr;
-    }
-
-    LottieLayer* layerByIdx(int16_t idx)
-    {
-        for (auto child = root->children.begin(); child < root->children.end(); ++child) {
-            auto layer = static_cast<LottieLayer*>(*child);
-            if (layer->idx == idx) return layer;
-        }
-        return nullptr;
+        return root->outFrame - root->inFrame;
     }
 
     LottieLayer* asset(unsigned long id)
@@ -671,7 +976,6 @@ struct LottieComposition
     char* version = nullptr;
     char* name = nullptr;
     float w, h;
-    float startFrame, endFrame;
     float frameRate;
     Array<LottieObject*> assets;
     Array<LottieInterpolator*> interpolators;
