@@ -36,14 +36,16 @@ public:
     }
 
     bool parse();
-    bool apply(LottieSlot* slot);
+    bool apply(LottieSlot* slot, bool byDefault);
     const char* sid(bool first = false);
+    void captureSlots(const char* key);
+    template<LottieProperty::Type type = LottieProperty::Type::Invalid> void registerSlot(LottieObject* obj, const char* sid);
 
     LottieComposition* comp = nullptr;
     const char* dirName = nullptr;       //base resource directory
+    char* slots = nullptr;
 
 private:
-    BlendMethod getBlendMethod();
     RGB24 getColor(const char *str);
     CompositeMethod getMatteType();
     FillRule getFillRule();
@@ -51,19 +53,20 @@ private:
     StrokeJoin getStrokeJoin();
     CompositeMethod getMaskMethod(bool inversed);
     LottieInterpolator* getInterpolator(const char* key, Point& in, Point& out);
-    uint8_t getDirection();
+    LottieEffect* getEffect(int type);
 
-    void getInperpolatorPoint(Point& pt);
+    void getInterpolatorPoint(Point& pt);
     void getPathSet(LottiePathSet& path);
     void getLayerSize(float& val);
-    void getValue(TextDocument& doc);
-    void getValue(PathSet& path);
-    void getValue(Array<Point>& pts);
-    void getValue(ColorStop& color);
-    void getValue(float& val);
-    void getValue(uint8_t& val);
-    void getValue(Point& pt);
-    void getValue(RGB24& color);
+    bool getValue(TextDocument& doc);
+    bool getValue(PathSet& path);
+    bool getValue(Array<Point>& pts);
+    bool getValue(ColorStop& color);
+    bool getValue(float& val);
+    bool getValue(uint8_t& val);
+    bool getValue(int8_t& val);
+    bool getValue(RGB24& color);
+    bool getValue(Point& pt);
 
     template<typename T> bool parseTangent(const char *key, LottieVectorFrame<T>& value);
     template<typename T> bool parseTangent(const char *key, LottieScalarFrame<T>& value);
@@ -74,8 +77,8 @@ private:
 
     LottieObject* parseObject();
     LottieObject* parseAsset();
-    LottieImage* parseImage(const char* data, const char* subPath, bool embedded);
-    LottieLayer* parseLayer();
+    void parseImage(LottieImage* image, const char* data, const char* subPath, bool embedded, float width, float height);
+    LottieLayer* parseLayer(LottieLayer* precomp);
     LottieObject* parseGroup();
     LottieRect* parseRect();
     LottieEllipse* parseEllipse();
@@ -87,27 +90,41 @@ private:
     LottiePolyStar* parsePolyStar();
     LottieRoundedCorner* parseRoundedCorner();
     LottieGradientFill* parseGradientFill();
-    LottieLayer* parseLayers();
+    LottieLayer* parseLayers(LottieLayer* root);
     LottieMask* parseMask();
     LottieTrimpath* parseTrimpath();
     LottieRepeater* parseRepeater();
+    LottieOffsetPath* parseOffsetPath();
     LottieFont* parseFont();
     LottieMarker* parseMarker();
 
+    void parseEffect(LottieEffect* effect, void(LottieParser::*func)(LottieEffect*, int));
+    void parseStroke(LottieEffect* effect, int idx);
+    void parseTritone(LottieEffect* effect, int idx);
+    void parseTint(LottieEffect* effect, int idx);
+    void parseFill(LottieEffect* effect, int idx);
+    void parseGaussianBlur(LottieEffect* effect, int idx);
+    void parseDropShadow(LottieEffect* effect, int idx);
+
+    bool parseDirection(LottieShape* shape, const char* key);
     bool parseCommon(LottieObject* obj, const char* key);
     void parseObject(Array<LottieObject*>& parent);
     void parseShapes(Array<LottieObject*>& parent);
     void parseText(Array<LottieObject*>& parent);
     void parseMasks(LottieLayer* layer);
+    void parseEffects(LottieLayer* layer);
     void parseTimeRemap(LottieLayer* layer);
     void parseStrokeDash(LottieStroke* stroke);
     void parseGradient(LottieGradient* gradient, const char* key);
+    void parseColorStop(LottieGradient* gradient);
     void parseTextRange(LottieText* text);
+    void parseTextAlignmentOption(LottieText* text);
     void parseAssets();
     void parseFonts();
-    void parseChars(Array<LottieGlyph*>& glyphes);
+    void parseChars(Array<LottieGlyph*>& glyphs);
     void parseMarkers();
-    void postProcess(Array<LottieGlyph*>& glyphes);
+    void parseEffect(LottieEffect* effect);
+    void postProcess(Array<LottieGlyph*>& glyphs);
 
     //Current parsing context
     struct Context {
