@@ -41,6 +41,12 @@ void GlRenderTask::run()
     // bind shader
     mProgram->load();
 
+    int32_t dLoc = mProgram->getUniformLocation("uDepth");
+    if (dLoc >= 0) {
+        // fixme: prevent compiler warning: macro expands to multiple statements [-Wmultistatement-macros]
+        GL_CHECK(glUniform1f(dLoc, mDrawDepth));
+    }
+
     // setup scissor rect
     GL_CHECK(glScissor(mViewport.x, mViewport.y, mViewport.w, mViewport.h));
 
@@ -97,6 +103,13 @@ void GlRenderTask::setDrawRange(uint32_t offset, uint32_t count)
 void GlRenderTask::setViewport(const RenderRegion &viewport)
 {
     mViewport = viewport;
+    if (mViewport.w < 0) {
+        mViewport.w = 0;
+    }
+
+    if (mViewport.h < 0) {
+        mViewport.h = 0;
+    }
 }
 
 GlStencilCoverTask::GlStencilCoverTask(GlRenderTask* stencil, GlRenderTask* cover, GlStencilMode mode)
@@ -116,10 +129,10 @@ void GlStencilCoverTask::run()
         GL_CHECK(glStencilFunc(GL_NOTEQUAL, 0x1, 0xFF));
         GL_CHECK(glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE));
     } else {
-        GL_CHECK(glStencilFuncSeparate(GL_FRONT, GL_ALWAYS, 0x1, 0xFF));
+        GL_CHECK(glStencilFuncSeparate(GL_FRONT, GL_ALWAYS, 0x0, 0xFF));
         GL_CHECK(glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP));
 
-        GL_CHECK(glStencilFuncSeparate(GL_BACK, GL_ALWAYS, 0x1, 0xFF));
+        GL_CHECK(glStencilFuncSeparate(GL_BACK, GL_ALWAYS, 0x0, 0xFF));
         GL_CHECK(glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP));
     }
     GL_CHECK(glColorMask(0, 0, 0, 0));
@@ -127,8 +140,8 @@ void GlStencilCoverTask::run()
     mStencilTask->run();
 
     if (mStencilMode == GlStencilMode::FillEvenOdd) {
-        GL_CHECK(glStencilFunc(GL_EQUAL, 0x01, 0x01));
-        GL_CHECK(glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE));
+        GL_CHECK(glStencilFunc(GL_NOTEQUAL, 0x00, 0x01));
+        GL_CHECK(glStencilOp(GL_REPLACE, GL_KEEP, GL_REPLACE));
     } else {
         GL_CHECK(glStencilFunc(GL_NOTEQUAL, 0x0, 0xFF));
         GL_CHECK(glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE));
@@ -138,20 +151,15 @@ void GlStencilCoverTask::run()
 
     mCoverTask->run();
 
-    if (mStencilMode == GlStencilMode::FillEvenOdd) {
-        GL_CHECK(glStencilFunc(GL_NOTEQUAL, 0x0, 0xFF));
-        GL_CHECK(glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE));
-
-        GL_CHECK(glColorMask(0, 0, 0, 0));
-
-        mStencilTask->run();
-
-        GL_CHECK(glColorMask(1, 1, 1, 1));
-    }
-
     GL_CHECK(glDisable(GL_STENCIL_TEST));
 }
 
+void GlStencilCoverTask::normalizeDrawDepth(int32_t maxDepth)
+{
+    mCoverTask->normalizeDrawDepth(maxDepth);
+    mStencilTask->normalizeDrawDepth(maxDepth);
+}
+
 GlComposeTask::GlComposeTask(GlProgram* program, GLuint target, GlRenderTarget* fbo, Array<GlRenderTask*>&& tasks)
  :GlRenderTask(program) ,mTargetFbo(target), mFbo(fbo), mTasks()
 {
@@ -171,19 +179,30 @@ GlComposeTask::~GlComposeTask()
 void GlComposeTask::run()
 {
     GL_CHECK(glBindFramebuffer(GL_FRAMEBUFFER, getSelfFbo()));
+    GL_CHECK(glViewport(0, 0, mRenderWidth, mRenderHeight));
+
+    GL_CHECK(glScissor(0, 0, mRenderWidth, mRenderHeight));
 
     // clear this fbo
     GL_CHECK(glClearColor(0, 0, 0, 0));
     GL_CHECK(glClearStencil(0));
-    GL_CHECK(glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT));
+    GL_CHECK(glClearDepthf(0.0));
+    GL_CHECK(glDepthMask(1));
+
+    GL_CHECK(glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
+    GL_CHECK(glDepthMask(0));
 
     for(uint32_t i = 0; i < mTasks.count; i++) {
         mTasks[i]->run();
     }
 
-    GLenum stencil_attachment = GL_STENCIL_ATTACHMENT;
-    GL_CHECK(glInvalidateFramebuffer(GL_FRAMEBUFFER, 1, &stencil_attachment));
-
+#if defined(THORVG_GL_TARGET_GLES)
+    // only OpenGLES has tiled base framebuffer and discard function
+    GLenum attachments[2] = {GL_STENCIL_ATTACHMENT, GL_DEPTH_ATTACHMENT };
+    GL_CHECK(glInvalidateFramebuffer(GL_FRAMEBUFFER, 2, attachments));
+#endif
+    // reset scissor box
+    GL_CHECK(glScissor(0, 0, mFbo->getWidth(), mFbo->getHeight()));
     onResolve();
 }
 
@@ -195,29 +214,31 @@ void GlComposeTask::onResolve() {
     GL_CHECK(glBindFramebuffer(GL_READ_FRAMEBUFFER, getSelfFbo()));
     GL_CHECK(glBindFramebuffer(GL_DRAW_FRAMEBUFFER, getResolveFboId()));
 
-    GL_CHECK(glBlitFramebuffer(0, 0, mFbo->getWidth(), mFbo->getHeight(), 0, 0, mFbo->getWidth(), mFbo->getHeight(), GL_COLOR_BUFFER_BIT, GL_NEAREST));
+
+    GL_CHECK(glBlitFramebuffer(0, 0, mRenderWidth, mRenderHeight, 0, 0, mRenderWidth, mRenderHeight, GL_COLOR_BUFFER_BIT, GL_NEAREST));
 }
 
 GlBlitTask::GlBlitTask(GlProgram* program, GLuint target, GlRenderTarget* fbo, Array<GlRenderTask*>&& tasks)
- : GlComposeTask(program, target, fbo, std::move(tasks))
+ : GlComposeTask(program, target, fbo, std::move(tasks)), mColorTex(fbo->getColorTexture())
 {
 }
 
-void GlBlitTask::setSize(uint32_t width, uint32_t height)
-{
-    mWidth = width;
-    mHeight = height;
-}
-
 void GlBlitTask::run()
 {
     GlComposeTask::run();
 
-    GL_CHECK(glScissor(0, 0, mWidth, mHeight));
     GL_CHECK(glBindFramebuffer(GL_FRAMEBUFFER, getTargetFbo()));
-    GL_CHECK(glBindFramebuffer(GL_READ_FRAMEBUFFER, getSelfFbo()));
+    GL_CHECK(glViewport(mTargetViewport.x, mTargetViewport.y, mTargetViewport.w, mTargetViewport.h));
 
-    GL_CHECK(glBlitFramebuffer(0, 0, mWidth, mHeight, 0, 0, mWidth, mHeight, GL_COLOR_BUFFER_BIT, GL_NEAREST));
+    GL_CHECK(glClearColor(0, 0, 0, 0));
+    GL_CHECK(glClear(GL_COLOR_BUFFER_BIT));
+
+    GL_CHECK(glDisable(GL_DEPTH_TEST));
+    // make sure the blending is correct
+    GL_CHECK(glEnable(GL_BLEND));
+    GL_CHECK(glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA));
+
+    GlRenderTask::run();
 }
 
 GlDrawBlitTask::GlDrawBlitTask(GlProgram* program, GLuint target, GlRenderTarget* fbo, Array<GlRenderTask*>&& tasks)
@@ -225,11 +246,138 @@ GlDrawBlitTask::GlDrawBlitTask(GlProgram* program, GLuint target, GlRenderTarget
 {
 }
 
+GlDrawBlitTask::~GlDrawBlitTask()
+{
+    if (mPrevTask) delete mPrevTask;
+}
+
 void GlDrawBlitTask::run()
 {
+    if (mPrevTask) mPrevTask->run();
+
     GlComposeTask::run();
 
     GL_CHECK(glBindFramebuffer(GL_FRAMEBUFFER, getTargetFbo()));
 
+    GL_CHECK(glViewport(0, 0, mParentWidth, mParentHeight));
+    GL_CHECK(glScissor(0, 0, mParentWidth, mParentWidth));
     GlRenderTask::run();
-}
\ No newline at end of file
+}
+
+GlClipTask::GlClipTask(GlRenderTask* clip, GlRenderTask* mask)
+ :GlRenderTask(nullptr), mClipTask(clip), mMaskTask(mask) {}
+
+GlClipTask::~GlClipTask()
+{
+    delete mClipTask;
+    delete mMaskTask;
+}
+
+void GlClipTask::run()
+{
+    GL_CHECK(glEnable(GL_STENCIL_TEST));
+    GL_CHECK(glColorMask(0, 0, 0, 0));
+    // draw clip path as normal stencil mask
+    GL_CHECK(glStencilFuncSeparate(GL_FRONT, GL_ALWAYS, 0x1, 0xFF));
+    GL_CHECK(glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP));
+
+    GL_CHECK(glStencilFuncSeparate(GL_BACK, GL_ALWAYS, 0x1, 0xFF));
+    GL_CHECK(glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP));
+
+    mClipTask->run();
+
+
+    // draw clip mask
+    GL_CHECK(glDepthMask(1));
+    GL_CHECK(glStencilFunc(GL_EQUAL, 0x0, 0xFF));
+    GL_CHECK(glStencilOp(GL_REPLACE, GL_KEEP, GL_REPLACE));
+
+    mMaskTask->run();
+
+    GL_CHECK(glColorMask(1, 1, 1, 1));
+    GL_CHECK(glDepthMask(0));
+    GL_CHECK(glDisable(GL_STENCIL_TEST));
+}
+
+void GlClipTask::normalizeDrawDepth(int32_t maxDepth)
+{
+    mClipTask->normalizeDrawDepth(maxDepth);
+    mMaskTask->normalizeDrawDepth(maxDepth);
+}
+
+GlSimpleBlendTask::GlSimpleBlendTask(BlendMethod method, GlProgram* program)
+ : GlRenderTask(program), mBlendMethod(method) {}
+
+void GlSimpleBlendTask::run()
+{
+    if (mBlendMethod == BlendMethod::Add) glBlendFunc(GL_ONE, GL_ONE);
+    else if (mBlendMethod == BlendMethod::Darken) {
+        glBlendFunc(GL_ONE, GL_ONE);
+        glBlendEquation(GL_MIN);
+    } else if (mBlendMethod == BlendMethod::Lighten) {
+        glBlendFunc(GL_ONE, GL_ONE);
+        glBlendEquation(GL_MAX);
+    }
+    else glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+
+    GlRenderTask::run();
+
+    if (mBlendMethod == BlendMethod::Darken || mBlendMethod == BlendMethod::Lighten) glBlendEquation(GL_FUNC_ADD);
+
+    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+}
+
+GlComplexBlendTask::GlComplexBlendTask(GlProgram* program, GlRenderTarget* dstFbo, GlRenderTarget* dstCopyFbo, GlRenderTask* stencilTask, GlComposeTask* composeTask)
+ : GlRenderTask(program), mDstFbo(dstFbo), mDstCopyFbo(dstCopyFbo), mStencilTask(stencilTask), mComposeTask(composeTask) {}
+
+GlComplexBlendTask::~GlComplexBlendTask()
+{
+    delete mStencilTask;
+    delete mComposeTask;
+}
+
+void GlComplexBlendTask::run()
+{
+    mComposeTask->run();
+
+    // copy the current fbo to the dstCopyFbo
+    GL_CHECK(glBindFramebuffer(GL_READ_FRAMEBUFFER, mDstFbo->getFboId()));
+    GL_CHECK(glBindFramebuffer(GL_DRAW_FRAMEBUFFER, mDstCopyFbo->getResolveFboId()));
+
+    GL_CHECK(glViewport(0, 0, mDstFbo->getViewport().w, mDstFbo->getViewport().h));
+    GL_CHECK(glScissor(0, 0, mDstFbo->getViewport().w, mDstFbo->getViewport().h));
+    
+    const auto& vp = getViewport();
+
+    GL_CHECK(glBlitFramebuffer(vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 0, 0, vp.w, vp.h, GL_COLOR_BUFFER_BIT, GL_LINEAR));
+
+    GL_CHECK(glBindFramebuffer(GL_FRAMEBUFFER, mDstFbo->getFboId()));
+
+    GL_CHECK(glEnable(GL_STENCIL_TEST));
+    GL_CHECK(glColorMask(0, 0, 0, 0));
+    GL_CHECK(glStencilFuncSeparate(GL_FRONT, GL_ALWAYS, 0x0, 0xFF));
+    GL_CHECK(glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP));
+
+    GL_CHECK(glStencilFuncSeparate(GL_BACK, GL_ALWAYS, 0x0, 0xFF));
+    GL_CHECK(glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP));
+
+
+    mStencilTask->run();
+
+    GL_CHECK(glColorMask(1, 1, 1, 1));
+    GL_CHECK(glStencilFunc(GL_NOTEQUAL, 0x0, 0xFF));
+    GL_CHECK(glStencilOp(GL_REPLACE, GL_KEEP, GL_REPLACE));
+
+    GL_CHECK(glBlendFunc(GL_ONE, GL_ZERO));
+
+    GlRenderTask::run();
+
+    GL_CHECK(glDisable(GL_STENCIL_TEST));
+    GL_CHECK(glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA));
+}
+
+void GlComplexBlendTask::normalizeDrawDepth(int32_t maxDepth)
+{
+    mStencilTask->normalizeDrawDepth(maxDepth);
+    GlRenderTask::normalizeDrawDepth(maxDepth);
+}
