@@ -20,6 +20,7 @@
  * SOFTWARE.
  */
 
+#include "tvgMath.h"
 #include "tvgGlTessellator.h"
 #include "tvgRender.h"
 #include "tvgGlList.h"
@@ -454,8 +455,8 @@ bool Edge::intersect(Edge *other, GlPoint *point)
 
     double scale = 1.0 / denom;
 
-    point->x = std::round(static_cast<float>(top->point.x - s_number * le_b * scale));
-    point->y = std::round(static_cast<float>(top->point.y + s_number * le_a * scale));
+    point->x = nearbyintf(static_cast<float>(top->point.x - s_number * le_b * scale));
+    point->y = nearbyintf(static_cast<float>(top->point.y + s_number * le_a * scale));
 
     if (std::isinf(point->x) || std::isinf(point->y)) {
         return false;
@@ -773,14 +774,14 @@ static int32_t _bezierCurveCount(const Bezier &curve)
     Bezier left{};
     Bezier right{};
 
-    bezSplit(curve, left, right);
+    curve.split(left, right);
 
     return _bezierCurveCount(left) + _bezierCurveCount(right);
 }
 
 static Bezier _bezFromArc(const GlPoint& start, const GlPoint& end, float radius) {
     // Calculate the angle between the start and end points
-    float angle = atan2(end.y - start.y, end.x - start.x);
+    float angle = tvg::atan2(end.y - start.y, end.x - start.x);
 
     // Calculate the control points of the cubic bezier curve
     float c = radius * 0.552284749831;  // c = radius * (4/3) * tan(pi/8)
@@ -979,7 +980,7 @@ void Tessellator::tessellate(const Array<const RenderShape *> &shapes)
 void Tessellator::visitShape(const PathCommand *cmds, uint32_t cmd_count, const Point *pts, uint32_t pts_count)
 {
     // all points at least need to be visit once
-    // so the points cound is at least is the same as the count in shape
+    // so the points count is at least the same as the count in shape
     resGlPoints->reserve(pts_count * 2);
     // triangle fans, the indices count is at least triangles number * 3
     resIndices->reserve((pts_count - 2) * 3);
@@ -1003,7 +1004,7 @@ void Tessellator::visitShape(const PathCommand *cmds, uint32_t cmd_count, const
                 pts++;
             } break;
             case PathCommand::CubicTo: {
-                // bezier curve needs to calcluate how many segment to split
+                // bezier curve needs to calculate how many segment to split
                 // for now just break curve into 16 segments for convenient
 
                 auto  last = outlines.last();
@@ -1023,7 +1024,7 @@ void Tessellator::visitShape(const PathCommand *cmds, uint32_t cmd_count, const
                 float step = 1.f / stepCount;
 
                 for (uint32_t s = 1; s < static_cast<uint32_t>(stepCount); s++) {
-                    last->append(pHeap->allocate<detail::Vertex>(detail::_upScalePoint(bezPointAt(curve, step * s))));
+                    last->append(pHeap->allocate<detail::Vertex>(detail::_upScalePoint(curve.at(step * s))));
                 }
 
                 last->append(pHeap->allocate<detail::Vertex>(detail::_upScalePoint(end)));
@@ -1093,7 +1094,7 @@ void Tessellator::mergeVertices()
         }
 
         if (v->point == v->prev->point) {
-            // merve v into v->prev
+            // merge v into v->prev
             while (auto e = v->edge_above.head) {
                 e->setBottom(v->prev);
             }
@@ -1638,30 +1639,130 @@ void Tessellator::emitTriangle(detail::Vertex *p1, detail::Vertex *p2, detail::V
 }
 
 
-Stroker::Stroker(Array<float> *points, Array<uint32_t> *indices) : mResGlPoints(points), mResIndices(indices)
+Stroker::Stroker(Array<float> *points, Array<uint32_t> *indices, const Matrix& matrix) : mResGlPoints(points), mResIndices(indices), mMatrix(matrix)
 {
 }
 
 void Stroker::stroke(const RenderShape *rshape)
 {
-    mMiterLimit = rshape->strokeMiterlimit() * 2.f;
-    mStrokeWidth = std::max(mStrokeWidth, rshape->strokeWidth());
+    mMiterLimit = rshape->strokeMiterlimit();
     mStrokeCap = rshape->strokeCap();
     mStrokeJoin = rshape->strokeJoin();
+    mStrokeWidth = rshape->strokeWidth();
 
+    if (isinf(mMatrix.e11)) {
+        auto strokeWidth = rshape->strokeWidth() * getScaleFactor(mMatrix);
+        if (strokeWidth <= MIN_GL_STROKE_WIDTH) strokeWidth = MIN_GL_STROKE_WIDTH;
+        mStrokeWidth = strokeWidth / mMatrix.e11;
+    }
 
     auto cmds = rshape->path.cmds.data;
     auto cmdCnt = rshape->path.cmds.count;
     auto pts = rshape->path.pts.data;
     auto ptsCnt = rshape->path.pts.count;
 
-    const float *dash_pattern = nullptr;
-    auto dash_count = rshape->strokeDash(&dash_pattern, nullptr);
+    if (rshape->strokeTrim()) {
+        auto begin = 0.0f;
+        auto end = 0.0f;
+        rshape->stroke->strokeTrim(begin, end);
 
-    if (dash_count == 0) {
-        doStroke(cmds, cmdCnt, pts, ptsCnt);
+        if (begin == end) return;
+
+        if (begin > end) {
+            doTrimStroke(cmds, cmdCnt, pts, ptsCnt, rshape->stroke->trim.simultaneous, begin, 1.0f);
+            doTrimStroke(cmds, cmdCnt, pts, ptsCnt, rshape->stroke->trim.simultaneous, 0.0f, end);
+        } else {
+            doTrimStroke(cmds, cmdCnt, pts, ptsCnt, rshape->stroke->trim.simultaneous, begin,  end);
+        }
+
+        return;
+    }
+
+    const float *dash_pattern = nullptr;
+    auto dashCnt = rshape->strokeDash(&dash_pattern, nullptr);
+
+    if (dashCnt == 0) doStroke(cmds, cmdCnt, pts, ptsCnt);
+    else doDashStroke(cmds, cmdCnt, pts, ptsCnt, dashCnt, dash_pattern);
+}
+
+
+RenderRegion Stroker::bounds() const
+{
+    return RenderRegion {
+        static_cast<int32_t>(floor(mLeftTop.x)),
+        static_cast<int32_t>(floor(mLeftTop.y)),
+        static_cast<int32_t>(ceil(mRightBottom.x - floor(mLeftTop.x))),
+        static_cast<int32_t>(ceil(mRightBottom.y - floor(mLeftTop.y))),
+    };
+}
+
+void Stroker::doTrimStroke(const PathCommand* cmds, uint32_t cmdCnt, const Point* pts, uint32_t ptsCnt, bool simultaneous, float start, float end)
+{
+    if (simultaneous) {
+        auto startCmds = cmds;
+        auto currCmds = cmds;
+        int ptsNum = 0;
+        for (uint32_t i = 0; i < cmdCnt; i++) {
+            switch (*currCmds) {
+                case PathCommand::MoveTo: {
+                    if (currCmds != startCmds) {
+                        PathTrim trim{};
+                        if (trim.trim(startCmds, currCmds - startCmds, pts, ptsNum, start, end)) {
+                            const auto& sCmds = trim.cmds();
+                            const auto& sPts = trim.pts();
+                            doStroke(sCmds.data, sCmds.count, sPts.data, sPts.count);
+                        }
+                        startCmds = currCmds;
+                        pts += ptsNum;
+                        ptsNum = 0;
+                    }
+                    currCmds++;
+                    ptsNum++;
+                    break;
+                }
+                case PathCommand::LineTo:
+                    currCmds++;
+                    ptsNum++;
+                    break;
+                case PathCommand::CubicTo:
+                    currCmds++;
+                    ptsNum += 3;
+                    break;
+                case PathCommand::Close: {
+                    PathTrim trim{};
+                    currCmds++;
+                    if (trim.trim(startCmds, currCmds - startCmds, pts, ptsNum, start, end)) {
+                        const auto& sCmds = trim.cmds();
+                        const auto& sPts = trim.pts();
+                        doStroke(sCmds.data, sCmds.count, sPts.data, sPts.count);
+                    }
+                    startCmds = currCmds;
+                    pts += ptsNum;
+                    ptsNum = 0;
+                    break;
+                }
+            }
+        }
+
+        if (startCmds != currCmds && ptsNum > 0) {
+            PathTrim trim{};
+
+            if (trim.trim(startCmds, currCmds - startCmds, pts, ptsNum, start, end)) {
+                const auto& sCmds = trim.cmds();
+                const auto& sPts = trim.pts();
+                doStroke(sCmds.data, sCmds.count, sPts.data, sPts.count);
+            }
+            startCmds = currCmds;
+            pts += ptsNum;
+            ptsNum = 0;
+        }
     } else {
-        doDashStroke(cmds, cmdCnt, pts, ptsCnt, dash_count, dash_pattern);
+        PathTrim trim{};
+        if (trim.trim(cmds, cmdCnt, pts, ptsCnt, start, end)) {
+            const auto& sCmds = trim.cmds();
+            const auto& sPts = trim.pts();
+            doStroke(sCmds.data, sCmds.count, sPts.data, sPts.count);
+        }
     }
 }
 
@@ -1670,42 +1771,46 @@ void Stroker::doStroke(const PathCommand *cmds, uint32_t cmd_count, const Point
     mResGlPoints->reserve(pts_count * 4 + 16);
     mResIndices->reserve(pts_count * 3);
 
+    auto validStrokeCap = false;
 
     for (uint32_t i = 0; i < cmd_count; i++) {
         switch (cmds[i]) {
             case PathCommand::MoveTo: {
-                if (mStrokeState.hasMove) {
+                if (validStrokeCap) { // check this, so we can skip if path only contains move instruction
                     strokeCap();
-                    mStrokeState.hasMove = false;
+                    validStrokeCap = false;
                 }
-                mStrokeState.hasMove = true;
                 mStrokeState.firstPt = *pts;
                 mStrokeState.firstPtDir = GlPoint{};
                 mStrokeState.prevPt = *pts;
                 mStrokeState.prevPtDir = GlPoint{};
                 pts++;
+                validStrokeCap = false;
             } break;
             case PathCommand::LineTo: {
+                validStrokeCap = true;
                 this->strokeLineTo(*pts);
                 pts++;
             } break;
             case PathCommand::CubicTo: {
+                validStrokeCap = true;
                 this->strokeCubicTo(pts[0], pts[1], pts[2]);
                 pts += 3;
             } break;
             case PathCommand::Close: {
                 this->strokeClose();
 
-                mStrokeState.hasMove = false;
+                validStrokeCap = false;
             } break;
             default:
                 break;
         }
     }
+    if (validStrokeCap) strokeCap();
 }
 
 void Stroker::doDashStroke(const PathCommand *cmds, uint32_t cmd_count, const Point *pts, uint32_t pts_count,
-                           uint32_t dast_count, const float *dash_pattern)
+                           uint32_t dash_count, const float *dash_pattern)
 {
     Array<PathCommand> dash_cmds{};
     Array<Point>       dash_pts{};
@@ -1713,7 +1818,7 @@ void Stroker::doDashStroke(const PathCommand *cmds, uint32_t cmd_count, const Po
     dash_cmds.reserve(20 * cmd_count);
     dash_pts.reserve(20 * pts_count);
 
-    DashStroke dash(&dash_cmds, &dash_pts, dast_count, dash_pattern);
+    DashStroke dash(&dash_cmds, &dash_pts, dash_count, dash_pattern);
 
     dash.doStroke(cmds, cmd_count, pts, pts_count);
 
@@ -1722,6 +1827,21 @@ void Stroker::doDashStroke(const PathCommand *cmds, uint32_t cmd_count, const Po
 
 void Stroker::strokeCap()
 {
+    if (mStrokeCap == StrokeCap::Butt) return;
+
+    if (mStrokeCap == StrokeCap::Square) {
+        if (mStrokeState.firstPt == mStrokeState.prevPt) strokeSquarePoint(mStrokeState.firstPt);
+        else {
+            strokeSquare(mStrokeState.firstPt, {-mStrokeState.firstPtDir.x, -mStrokeState.firstPtDir.y});
+            strokeSquare(mStrokeState.prevPt, mStrokeState.prevPtDir);
+        }
+    } else if (mStrokeCap == StrokeCap::Round) {
+        if (mStrokeState.firstPt == mStrokeState.prevPt) strokeRoundPoint(mStrokeState.firstPt);
+        else {
+            strokeRound(mStrokeState.firstPt, {-mStrokeState.firstPtDir.x, -mStrokeState.firstPtDir.y});
+            strokeRound(mStrokeState.prevPt, mStrokeState.prevPtDir);
+        }
+    }
 }
 
 void Stroker::strokeLineTo(const GlPoint &curr)
@@ -1773,6 +1893,16 @@ void Stroker::strokeLineTo(const GlPoint &curr)
         mStrokeState.prevPtDir = dir;
         mStrokeState.prevPt = curr;
     }
+
+    if (ia == 0) {
+        mRightBottom.x = mLeftTop.x = curr.x;
+        mRightBottom.y = mLeftTop.y = curr.y;
+    }
+
+    mLeftTop.x = std::min(mLeftTop.x, min(min(a.x, b.x), min(c.x, d.x)));
+    mLeftTop.y = std::min(mLeftTop.y, min(min(a.y, b.y), min(c.y, d.y)));
+    mRightBottom.x = std::max(mRightBottom.x, max(max(a.x, b.x), max(c.x, d.x)));
+    mRightBottom.y = std::max(mRightBottom.y, max(max(a.y, b.y), max(c.y, d.y)));
 }
 
 void Stroker::strokeCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlPoint &end)
@@ -1783,11 +1913,18 @@ void Stroker::strokeCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlPo
     curve.ctrl2 = Point{cnt2.x, cnt2.y};
     curve.end = Point{end.x, end.y};
 
-    auto count = detail::_bezierCurveCount(curve);
+    Bezier relCurve {curve.start, curve.ctrl1, curve.ctrl2, curve.end};
+    relCurve.start *= mMatrix;
+    relCurve.ctrl1 *= mMatrix;
+    relCurve.ctrl2 *= mMatrix;
+    relCurve.end *= mMatrix;
+
+    auto count = detail::_bezierCurveCount(relCurve);
+
     float step = 1.f / count;
 
     for (int32_t i = 0; i <= count; i++) {
-        strokeLineTo(bezPointAt(curve, step * i));
+        strokeLineTo(curve.at(step * i));
     }
 }
 
@@ -1799,8 +1936,6 @@ void Stroker::strokeClose()
 
     // join firstPt with prevPt
     this->strokeJoin(mStrokeState.firstPtDir);
-
-    mStrokeState.hasMove = false;
 }
 
 void Stroker::strokeJoin(const GlPoint &dir)
@@ -1861,6 +1996,11 @@ void Stroker::strokeRound(const GlPoint &prev, const GlPoint &curr, const GlPoin
         return;
     }
 
+    mLeftTop.x = std::min(mLeftTop.x, min(center.x, min(prev.x, curr.x)));
+    mLeftTop.y = std::min(mLeftTop.y, min(center.y, min(prev.y, curr.y)));
+    mRightBottom.x = std::max(mRightBottom.x, max(center.x, max(prev.x, curr.x)));
+    mRightBottom.y = std::max(mRightBottom.y, max(center.y, max(prev.y, curr.y)));
+
     // Fixme: just use bezier curve to calculate step count
     auto count = detail::_bezierCurveCount(detail::_bezFromArc(prev, curr, strokeRadius()));
 
@@ -1888,9 +2028,42 @@ void Stroker::strokeRound(const GlPoint &prev, const GlPoint &curr, const GlPoin
         this->mResIndices->push(oi);
 
         pi = oi;
+
+        mLeftTop.x = std::min(mLeftTop.x, out.x);
+        mLeftTop.y = std::min(mLeftTop.y, out.y);
+        mRightBottom.x = std::max(mRightBottom.x, out.x);
+        mRightBottom.y = std::max(mRightBottom.y, out.y);
     }
 }
 
+
+void Stroker::strokeRoundPoint(const GlPoint &p)
+{
+    // Fixme: just use bezier curve to calculate step count
+    auto count = detail::_bezierCurveCount(detail::_bezFromArc(p, p, strokeRadius())) * 2;
+    auto c = detail::_pushVertex(mResGlPoints, p.x, p.y);
+    auto step = 2 * M_PI / (count - 1);
+
+    for (uint32_t i = 1; i <= static_cast<uint32_t>(count); i++) {
+        float angle = i * step;
+        GlPoint dir = {cos(angle), sin(angle)};
+        GlPoint out = p + dir * strokeRadius();
+        auto oi = detail::_pushVertex(mResGlPoints, out.x, out.y);
+
+        if (oi > 1) {
+            mResIndices->push(c);
+            mResIndices->push(oi);
+            mResIndices->push(oi - 1);
+        }
+    }
+
+    mLeftTop.x = std::min(mLeftTop.x, p.x - strokeRadius());
+    mLeftTop.y = std::min(mLeftTop.y, p.y - strokeRadius());
+    mRightBottom.x = std::max(mRightBottom.x, p.x + strokeRadius());
+    mRightBottom.y = std::max(mRightBottom.y, p.y + strokeRadius());
+}
+
+
 void Stroker::strokeMiter(const GlPoint &prev, const GlPoint &curr, const GlPoint &center)
 {
     auto pp1 = prev - center;
@@ -1902,7 +2075,7 @@ void Stroker::strokeMiter(const GlPoint &prev, const GlPoint &curr, const GlPoin
 
     auto pe = out * k;
 
-    if (detail::_pointLength(pe) >= mMiterLimit) {
+    if (detail::_pointLength(pe) >= mMiterLimit * strokeRadius()) {
         this->strokeBevel(prev, curr, center);
         return;
     }
@@ -1923,8 +2096,15 @@ void Stroker::strokeMiter(const GlPoint &prev, const GlPoint &curr, const GlPoin
     this->mResIndices->push(e);
     this->mResIndices->push(cp2);
     this->mResIndices->push(c);
+
+    mLeftTop.x = std::min(mLeftTop.x, join.x);
+    mLeftTop.y = std::min(mLeftTop.y, join.y);
+
+    mRightBottom.x = std::max(mRightBottom.x, join.x);
+    mRightBottom.y = std::max(mRightBottom.y, join.y);
 }
 
+
 void Stroker::strokeBevel(const GlPoint &prev, const GlPoint &curr, const GlPoint &center)
 {
     auto a = detail::_pushVertex(mResGlPoints, prev.x, prev.y);
@@ -1936,6 +2116,80 @@ void Stroker::strokeBevel(const GlPoint &prev, const GlPoint &curr, const GlPoin
     mResIndices->push(c);
 }
 
+
+void Stroker::strokeSquare(const GlPoint& p, const GlPoint& outDir)
+{
+    GlPoint normal{-outDir.y, outDir.x};
+
+    GlPoint a = p + normal * strokeRadius();
+    GlPoint b = p - normal * strokeRadius();
+    GlPoint c = a + outDir * strokeRadius();
+    GlPoint d = b + outDir * strokeRadius();
+
+
+    auto ai = detail::_pushVertex(mResGlPoints, a.x, a.y);
+    auto bi = detail::_pushVertex(mResGlPoints, b.x, b.y);
+    auto ci = detail::_pushVertex(mResGlPoints, c.x, c.y);
+    auto di = detail::_pushVertex(mResGlPoints, d.x, d.y);
+
+    mResIndices->push(ai);
+    mResIndices->push(bi);
+    mResIndices->push(ci);
+
+    mResIndices->push(ci);
+    mResIndices->push(bi);
+    mResIndices->push(di);
+
+    mLeftTop.x = std::min(mLeftTop.x, min(min(a.x, b.x), min(c.x, d.x)));
+    mLeftTop.y = std::min(mLeftTop.y, min(min(a.y, b.y), min(c.y, d.y)));
+    mRightBottom.x = std::max(mRightBottom.x, max(max(a.x, b.x), max(c.x, d.x)));
+    mRightBottom.y = std::max(mRightBottom.y, max(max(a.y, b.y), max(c.y, d.y)));
+}
+
+
+void Stroker::strokeSquarePoint(const GlPoint& p)
+{
+    auto offsetX = Point{strokeRadius(), 0.0f};
+    auto offsetY = Point{0.0f, strokeRadius()};
+
+    auto a = p + offsetX + offsetY;
+    auto b = p - offsetX + offsetY;
+    auto c = p - offsetX - offsetY;
+    auto d = p + offsetX - offsetY;
+
+    auto ai = detail::_pushVertex(mResGlPoints, a.x, a.y);
+    auto bi = detail::_pushVertex(mResGlPoints, b.x, b.y);
+    auto ci = detail::_pushVertex(mResGlPoints, c.x, c.y);
+    auto di = detail::_pushVertex(mResGlPoints, d.x, d.y);
+
+    mResIndices->push(ai);
+    mResIndices->push(bi);
+    mResIndices->push(ci);
+
+    mResIndices->push(ci);
+    mResIndices->push(di);
+    mResIndices->push(ai);
+
+    mLeftTop.x = std::min(mLeftTop.x, std::min(std::min(a.x, b.x), std::min(c.x, d.x)));
+    mLeftTop.y = std::min(mLeftTop.y, std::min(std::min(a.y, b.y), std::min(c.y, d.y)));
+    mRightBottom.x = std::max(mRightBottom.x, std::max(std::max(a.x, b.x), std::max(c.x, d.x)));
+    mRightBottom.y = std::max(mRightBottom.y, std::max(std::max(a.y, b.y), std::max(c.y, d.y)));
+}
+
+
+void Stroker::strokeRound(const GlPoint& p, const GlPoint& outDir)
+{
+    GlPoint normal{-outDir.y, outDir.x};
+
+    GlPoint a = p + normal * strokeRadius();
+    GlPoint b = p - normal * strokeRadius();
+    GlPoint c = p + outDir * strokeRadius();
+
+    strokeRound(a, c, p);
+    strokeRound(c, b, p);
+}
+
+
 DashStroke::DashStroke(Array<PathCommand> *cmds, Array<Point> *pts, uint32_t dash_count, const float *dash_pattern)
     : mCmds(cmds),
       mPts(pts),
@@ -1949,6 +2203,7 @@ DashStroke::DashStroke(Array<PathCommand> *cmds, Array<Point> *pts, uint32_t das
 {
 }
 
+
 void DashStroke::doStroke(const PathCommand *cmds, uint32_t cmd_count, const Point *pts, uint32_t pts_count)
 {
     for (uint32_t i = 0; i < cmd_count; i++) {
@@ -1985,6 +2240,7 @@ void DashStroke::doStroke(const PathCommand *cmds, uint32_t cmd_count, const Poi
     }
 }
 
+
 void DashStroke::dashLineTo(const GlPoint &to)
 {
     float len = detail::_pointLength(mPtCur - to);
@@ -2032,6 +2288,7 @@ void DashStroke::dashLineTo(const GlPoint &to)
     mPtCur = to;
 }
 
+
 void DashStroke::dashCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlPoint &end)
 {
     Bezier cur;
@@ -2040,7 +2297,7 @@ void DashStroke::dashCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlP
     cur.ctrl2 = Point{cnt2.x, cnt2.y};
     cur.end = Point{end.x, end.y};
 
-    auto len = bezLength(cur);
+    auto len = cur.length();
 
     if (len < mCurrLen) {
         mCurrLen -= len;
@@ -2054,7 +2311,7 @@ void DashStroke::dashCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlP
 
             Bezier left, right;
 
-            bezSplitAt(cur, mCurrLen, left, right);
+            cur.split(mCurrLen, left, right);
 
             if (mCurrIdx == 0) {
                 this->moveTo(left.start);
@@ -2084,18 +2341,21 @@ void DashStroke::dashCubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlP
     mPtCur = end;
 }
 
+
 void DashStroke::moveTo(const GlPoint &pt)
 {
     mPts->push(Point{pt.x, pt.y});
     mCmds->push(PathCommand::MoveTo);
 }
 
+
 void DashStroke::lineTo(const GlPoint &pt)
 {
     mPts->push(Point{pt.x, pt.y});
     mCmds->push(PathCommand::LineTo);
 }
 
+
 void DashStroke::cubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlPoint &end)
 {
     mPts->push(Point{cnt1.x, cnt1.y});
@@ -2105,11 +2365,231 @@ void DashStroke::cubicTo(const GlPoint &cnt1, const GlPoint &cnt2, const GlPoint
 }
 
 
+bool PathTrim::trim(const PathCommand* cmds, uint32_t cmd_count, const Point* pts, uint32_t pts_count, float start, float end)
+{
+    if (end - start < 0.0001f) return false;
+
+    auto len = pathLength(cmds, cmd_count, pts, pts_count);
+    if (len < 0.001f) return false;
+
+    auto startLength = len * start;
+    auto endLength = len * end;
+    if (startLength >= endLength) return false;
+
+    trimPath(cmds, cmd_count, pts, pts_count, startLength, endLength);
+
+    return true;
+}
+
+
+float PathTrim::pathLength(const PathCommand* cmds, uint32_t cmd_count, const Point* pts, uint32_t pts_count)
+{
+    auto len = 0.0f;
+    Point zero = {0.0f, 0.0f};
+    const Point* prev = nullptr;
+    const Point* begin = nullptr;
+
+    for (uint32_t i = 0; i < cmd_count; i++) {
+        switch (cmds[i]) {
+            case PathCommand::MoveTo: {
+                prev = pts;
+                begin = pts;
+                pts++;
+                break;
+            }
+
+            case PathCommand::LineTo: {
+                if (prev != nullptr) len += length(prev, pts);
+                if (begin == nullptr) begin = pts;
+                prev = pts;
+                pts++;
+                break;
+            }
+
+            case PathCommand::CubicTo: {
+                if (prev == nullptr || begin == nullptr) {
+                    prev = begin = &zero;
+                }
+                Bezier b{ *prev, pts[0], pts[1], pts[2]};
+                len += b.length();
+                prev = pts + 2;
+                pts += 3;
+                break;
+            }
+
+            case PathCommand::Close: {
+                if (prev != nullptr && begin != nullptr && prev != begin) {
+                    len += length(prev, begin);
+                }
+                prev = begin = nullptr;
+                break;
+            }
+        }
+    }
+    return len;
+}
+
+
+void PathTrim::trimPath(const PathCommand* cmds, uint32_t cmd_count, const Point* pts, uint32_t pts_count, float start, float end)
+{
+    auto pos = 0.0f;
+    Point zero = {0.0f, 0.0f};
+    Point prev = {};
+    Point begin = {};
+    auto closed = true;
+    auto pushedMoveTo = false;
+    auto hasLineTo = false;
+
+    auto handle_line_to = [&](const Point* p1, const Point* p2) {
+        auto currLen = length(p1, p2);
+        if (pos + currLen <= start) {
+            pos += currLen;
+            prev = *p2;
+            return;
+        }
+
+        if (pos >= end)  {
+            prev = *p2;
+            return;
+        }
+
+        Line line{*p1, *p2};
+
+        if (pos < start) {
+            Line left, right;
+            line.split(start - pos, left, right);
+
+            pos += left.length();
+            line = right;
+        }
+
+        if (pos + currLen > end) {
+            Line left, right;
+            line.split(end - pos, left, right);
+            pos += left.length();
+            line = left;
+        }
+
+        if (!pushedMoveTo) {
+            mCmds.push(PathCommand::MoveTo);
+            mPts.push(line.pt1);
+            pushedMoveTo = true;
+            begin = line.pt1;
+        }
+
+        pos += line.length();
+
+        mCmds.push(PathCommand::LineTo);
+        mPts.push(line.pt2);
+        prev = line.pt2;
+        hasLineTo = true;
+        closed = false;
+    };
+
+    for (uint32_t i = 0; i < cmd_count; i++) {
+        if (pos - end > 0.001f) return; // we are done
+
+        if (pos >= end) return;
+
+        switch (cmds[i]) {
+            case PathCommand::MoveTo: {
+                prev = *pts;
+                begin = *pts;
+                pts++;
+                closed = false;
+                break;
+            }
+            case PathCommand::LineTo: {
+                handle_line_to(&prev, pts);
+                hasLineTo = true;
+                pts++;
+                break;
+            }
+            case PathCommand::CubicTo: {
+                Bezier b{ prev, pts[0], pts[1], pts[2]};
+
+                auto currLen = b.length();
+                if (pos + currLen <= start) {
+                    pos += currLen;
+                    prev = pts[2];
+                    pts += 3;
+                    break;
+                }
+
+                if (pos < start) {
+                    Bezier left, right;
+                    b.split(start - pos, left, right);
+                    pos += left.length();
+                    b = right;
+                }
+
+                if (pos + currLen > end) {
+                    Bezier left, right;
+                    b.split(end - pos, left, right);
+                    pos += left.length();
+                    b = left;
+                }
+
+                if (!pushedMoveTo) {
+                    mCmds.push(PathCommand::MoveTo);
+                    mPts.push(b.start);
+                    pushedMoveTo = true;
+                }
+
+
+                pos += b.length();
+                mCmds.push(PathCommand::CubicTo);
+                mPts.push(b.ctrl1);
+                mPts.push(b.ctrl2);
+                mPts.push(b.end);
+                prev = b.end;
+                hasLineTo = true;
+                closed = false;
+                pts += 3;
+                break;
+            }
+            case PathCommand::Close: {
+                if (closed) break;
+                if (!hasLineTo) {
+                    closed = true;
+                    pushedMoveTo = false;
+                    prev = begin = zero;
+                    break;
+                }
+                if (prev == begin) {
+                    prev = begin = zero;
+                    closed = true;
+                    pushedMoveTo = false;
+                    break;
+                }
+                auto currLen = length(&prev, &begin);
+                if (currLen + pos < start) {
+                    pos += currLen;
+                    break;
+                }
+                if (pos + currLen  <= end) {
+                    mCmds.push(PathCommand::Close);
+                    pos += currLen;
+                    break;
+                }
+                // handle_line_to(&prev, &begin);
+                closed = true;
+                pushedMoveTo = false;
+                prev = begin = zero;
+                pos += currLen;
+                break;
+            }
+        }
+    }
+}
+
+
 BWTessellator::BWTessellator(Array<float>* points, Array<uint32_t>* indices): mResPoints(points), mResIndices(indices)
 {
 }
 
-void BWTessellator::tessellate(const RenderShape *rshape)
+
+void BWTessellator::tessellate(const RenderShape *rshape, const Matrix& matrix)
 {
     auto cmds = rshape->path.cmds.data;
     auto cmdCnt = rshape->path.cmds.count;
@@ -2147,13 +2627,20 @@ void BWTessellator::tessellate(const RenderShape *rshape)
             case PathCommand::CubicTo: {
                 Bezier curve{pts[-1], pts[0], pts[1], pts[2]};
 
-                auto stepCount = detail::_bezierCurveCount(curve);
+                Bezier relCurve {pts[-1], pts[0], pts[1], pts[2]};
+                relCurve.start *= matrix;
+                relCurve.ctrl1 *= matrix;
+                relCurve.ctrl2 *= matrix;
+                relCurve.end *= matrix;
+
+                auto stepCount = detail::_bezierCurveCount(relCurve);
+
                 if (stepCount <= 1) stepCount = 2;
 
                 float step = 1.f / stepCount;
 
-                for (uint32_t s = 1; s < static_cast<uint32_t>(stepCount); s++) {
-                    auto pt = bezPointAt(curve, step * s);
+                for (uint32_t s = 1; s <= static_cast<uint32_t>(stepCount); s++) {
+                    auto pt = curve.at(step * s);
                     auto currIndex = pushVertex(pt.x, pt.y);
 
                     if (prevIndex == 0) {
@@ -2174,11 +2661,36 @@ void BWTessellator::tessellate(const RenderShape *rshape)
     }
 }
 
+
+RenderRegion BWTessellator::bounds() const
+{
+    return RenderRegion {
+        static_cast<int32_t>(floor(mLeftTop.x)),
+        static_cast<int32_t>(floor(mLeftTop.y)),
+        static_cast<int32_t>(ceil(mRightBottom.x - floor(mLeftTop.x))),
+        static_cast<int32_t>(ceil(mRightBottom.y - floor(mLeftTop.y))),
+    };
+}
+
+
 uint32_t BWTessellator::pushVertex(float x, float y)
 {
-    return detail::_pushVertex(mResPoints, x, y);
+    auto index = detail::_pushVertex(mResPoints, x, y);
+
+    if (index == 0) {
+        mRightBottom.x = mLeftTop.x = x;
+        mRightBottom.y = mLeftTop.y = y;
+    } else {
+        mLeftTop.x = min(mLeftTop.x, x);
+        mLeftTop.y = min(mLeftTop.y, y);
+        mRightBottom.x = max(mRightBottom.x, x);
+        mRightBottom.y = max(mRightBottom.y , y);
+    }
+
+    return index;
 }
 
+
 void BWTessellator::pushTriangle(uint32_t a, uint32_t b, uint32_t c)
 {
     mResIndices->push(a);
